<?php


namespace App\Mobile\Synchronization\Controllers;


use App\Http\Controller;
use App\Mobile\Synchronization\Requests\SyncRouteRequest;
use App\Mobile\Synchronization\Resources\SyncClientResource;
use Schaffen\Financial\BillingRoute\Repositories\RouteRepository;
use Schaffen\Financial\FinancialMovement\Models\FinancialMovement;
use Schaffen\Financial\FinancialMovement\Repositories\FinancialMovementRepository;
use Schaffen\Financial\FinancialMovement\Services\CalculationHandle;

class SyncChargeController extends Controller
{
    private FinancialMovementRepository $financialMovementRepository;
    private RouteRepository $routeRepository;

    public function __construct(
        FinancialMovementRepository $financialMovementRepository,
        RouteRepository $routeRepository
    )
    {
        $this->financialMovementRepository = $financialMovementRepository;
        $this->routeRepository = $routeRepository;
    }

    public function sync(SyncRouteRequest $request)
    {
        $input = $request->validated();
        $route = $this->routeRepository->find($input['route_id'], ['clients']);
        $vencimento = now()->endOfMonth()->endOfDay();
        $mvfs = $this->financialMovementRepository->list([
            "mvf_status" => FinancialMovement::STATUS_ABERTO,
            "pes_codigo_in" => $route->clients->pluck('pes_codigo'),
            "par_vencimento_end" => $vencimento,

        ],["installment" => function($query) use($vencimento) {
            $query->where('par_data_vencimento_atual', '<=', $vencimento);
            return $query;
        }]);

        $mvfs->each(function ($mvf){
            $mvf->installment->each(function (&$installment) use($mvf){
                $installment->total = CalculationHandle::installmentTotal($installment, $mvf);
            });
        });
        $clients = $route->clients->map(function ($client) use($mvfs){
            $client->mvfs = $mvfs->filter(function ($mvf, $key) use ($client){
                return $mvf->mvf_fk_pes_codigo == $client->pes_codigo;
            });
            return $client;
        })->filter(function ($client, $key){
            return count($client->mvfs) > 0;
        });
        //dd($clients->toArray());
        return SyncClientResource::collection($clients);
    }

    public function uploadSync(Request $request)
    {

    }
}