<?php

namespace App\RestApi\Person\Requests;

use Freelabois\LaravelQuickstart\Traits\RequestValidator;
use Illuminate\Foundation\Http\FormRequest;

class PersonRequest extends FormRequest
{
    use RequestValidator;

    public array $post_rules = [
        'pes_nome' => 'required|string|max:80',
        'pes_identificador' => 'required|string|max:15|identification|nullable',
        'pes_tipo' => 'integer|min:0|max:2|nullable',
        'pes_razao_social' => 'string|max:80|nullable',
        'pes_codigo_externo' => 'string|max:30|nullable',
        'pes_inscricao_estadual' => 'string|max:30|nullable',
        'pes_latitude' => 'numeric|nullable',
        'pes_longitude' => 'numeric|nullable',
        'pes_nome_reduzido' => 'string|max:30|nullable',
        'pes_fk_fil_codigo' => 'integer|nullable|exists:dynamicdb.cadastro.filial,fil_codigo',
        'pes_observacao' => 'string|nullable',
        'pes_limite_credito' => 'numeric|nullable',
        'pes_fk_reg_codigo' => 'integer|nullable|exists:dynamicdb.cadastro.regiao,reg_codigo',
        'pes_fk_seg_codigo' => 'integer|nullable|exists:dynamicdb.cadastro.segmento,seg_codigo',
        'pes_status' => 'integer|nullable|min:0|max:1',
        'pes_cad_pro' => 'string|max:30|nullable',
        'pes_fiscal_iss_retido' => 'integer|nullable|min:0|max:1',
        'pes_fiscal_indicador_ie' => 'integer|nullable',
        'pes_inscricao_municipal' => 'string|max:30|nullable',
        'pes_limite_bloqueio' => 'integer|nullable|min:0|max:1',
        'pes_limite_observacao' => 'string|max:100|nullable',
        'pes_representante_legal' => 'string|max:80|nullable',
        'pes_representante_cpf' => 'string|max:15|nullable',
        'pes_representante_observacao' => 'string|max:100|nullable',
        'pes_fiscal_tipo_cliente' => 'integer|min:1|max:2|nullable',
        'pes_dias_bloqueio_os' => 'integer|nullable',
        'pes_envia_email' => 'integer|nullable',
        'pes_limite_emprestimo' => 'integer|nullable',
        'pes_limite_documentos' => 'integer|nullable',
        'pes_limite_docto_teto' => 'integer|nullable',
        'pes_limite_docto_taxa' => 'integer|nullable',
        'pes_limite_docto_data_aprov' => 'string|date_format:Y-m-d H:i:s|nullable',
        'pes_limite_docto_data_venc' => 'string|date_format:Y-m-d H:i:s|nullable',
        //Validação do endereço
        'address.pee_logradouro' => 'required|string|max:80|nullable',
        'address.pee_numero' => 'required|string|max:10|nullable',
        'address.pee_bairro' => 'required|string|max:80|nullable',
        'address.pee_cep' => 'required|string|max:10|nullable',
        'address.pee_pais' => 'required|string|max:30|nullable',
        'address.pee_cidade' => 'required|string|max:80|nullable',
        'address.pee_uf' => 'required|string|max:2|nullable',
        'address.pee_complemento' => 'required|string|max:80|nullable',
        'address.pee_latitude' => 'required_with:pee_longitude|numeric|nullable',
        'address.pee_longitude' => 'required_with:pee_latitude|numeric|nullable',
        'address.pee_fk_tpe_codigo' => 'required|integer|nullable|exists:dynamicdb.cadastro.tipo_endereco,tpe_codigo',
        //Validação dos dados de contato
        'contact.peco_contato' => 'required|string|max:80',
        'contact.peco_email' => 'required_without:contact.peco_telefone|string|max:250|email',
        'contact.peco_telefone' => 'required_without:contact.peco_email|string|max:15'
    ];

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

}
